import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:intl/intl.dart';
import 'package:jiosaavn_wrapper/enum/artist_type.dart';
import 'package:jiosaavn_wrapper/jiosaavn_wrapper.dart';
import 'package:jiosaavn_wrapper/modals/artist.dart';
import 'package:jiosaavn_wrapper/modals/playlist.dart';
import 'package:jiosaavn_wrapper/modals/search_result.dart';
import 'package:jiosaavn_wrapper/modals/song.dart';
import 'package:jiosaavn_wrapper/utilities/extensions.dart';
import 'package:http/http.dart' as http;
import 'package:flutter_cache/flutter_cache.dart' as cache;

class PluginFactory {
  static SearchResult createSearchResult(Map<String, dynamic> map) {
    return SearchResult(
      artists: PluginFactory.createArtistSearchResult(map['artists']['data']),
      albums: PluginFactory.createAlbumSearchResult(map['albums']['data']),
      songs: PluginFactory.createSongSearchResult(map['songs']['data']),
      playlists:
          PluginFactory.createPlaylistSearchResult(map['playlists']['data']),
      topQuery: PluginFactory.createTopSearchResult(map['topquery']['data']),
    );
  }

  static List<ArtistSearchResult> createArtistSearchResult(
      List<Map<String, dynamic>> map) {
    return [
      ...map.map((artist) {
        return ArtistSearchResult(
          id: artist['id'],
          title: artist['title'],
          description: artist['description'],
          type: SearchResultType.artist,
          image: artist['image'],
        );
      })
    ];
  }

  static List<AlbumSearchResult> createAlbumSearchResult(
      List<Map<String, dynamic>> map) {
    return [
      ...map.map((album) {
        return AlbumSearchResult(
          id: album['id'],
          title: album['title'],
          subtitle: album['subtitle'],
          description: album['description'],
          type: SearchResultType.album,
          image: album['image'],
          permaURL: album['perma_url'],
          pIds: album['more_info']['song_pids'],
          year: int.parse(album['more_info']['year']),
        );
      })
    ];
  }

  static List<SongSearchResult> createSongSearchResult(
      List<Map<String, dynamic>> map) {
    return [
      ...map.map((song) {
        return SongSearchResult(
          id: song['id'],
          title: song['title'],
          subtitle: song['subtitle'],
          description: song['description'],
          type: SearchResultType.song,
          image: song['image'],
          permaURL: song['perma_url'],
          album: song['more_info']['album'],
          primaryArtists: song['more_info']['primary_artists'],
          singers: song['more_info']['singers'],
          previewMediaURL: song['more_info']['vlink'],
        );
      })
    ];
  }

  static List<PlaylistSearchResult> createPlaylistSearchResult(
      List<Map<String, dynamic>> map) {
    return [
      ...map.map((playlist) {
        return PlaylistSearchResult(
          id: playlist['id'],
          title: playlist['title'],
          subtitle: playlist['subtitle'],
          type: SearchResultType.playlist,
          image: playlist['image'],
          permaURL: playlist['perma_url'],
          description: playlist['description'],
        );
      })
    ];
  }

  static List createTopSearchResult(List<Map<String, dynamic>> map) {
    return [
      ...map.map((topresult) {
        final type = SearchResultType.values.firstWhere(
            (element) => describeEnum(element) == topresult['type']);
        switch (type) {
          case SearchResultType.album:
            return PluginFactory.createAlbumSearchResult([topresult]).first;
          case SearchResultType.artist:
            return PluginFactory.createArtistSearchResult([topresult]).first;
          case SearchResultType.playlist:
            return PluginFactory.createPlaylistSearchResult([topresult]).first;
          default:
            return PluginFactory.createSongSearchResult([topresult]).first;
        }
      })
    ];
  }

  static Future<Playlist> createPlayList(Map<String, dynamic> map) async {
    final moreInfo = map['more_info'] as Map<String, dynamic>;
    List<Song> songs = [];
    for (var song in map['list']) {
      songs.add(await PluginFactory.createSong(song));
    }
    return Playlist(
      id: map['id'],
      title: (map['title'] as String).sanitize,
      subtitle: (map['subtitle'] as String).sanitize,
      description: (map['header_desc'] as String).sanitize,
      permaURL: map['perma_url'],
      image: map['image'],
      totalSongs: int.parse(map['list_count']),
      followers: int.parse(moreInfo['follower_count']),
      songs: songs,
    );
  }

  static Future<Song> createSong(Map<String, dynamic> map) async {
    final moreInfo = map['more_info'] as Map<String, dynamic>;
    List<Artist> allArtists = [];
    (moreInfo['artistMap'] as Map<String, List<Map<String, dynamic>>>)
        .forEach((key, value) {
      for (var item in value) {
        allArtists.add(PluginFactory.createArtist(item));
      }
    });
    bool hasLyrics = moreInfo['has_lyrics'].toString() == 'true';
    return Song(
      id: map['id'],
      albumId: moreInfo['album_id'],
      album: moreInfo['album'],
      label: moreInfo['label'],
      title: (map['title'] as String).sanitize,
      subtitle: (map['subtitle'] as String).sanitize,
      lowResImage: (map['image'] as String).lowRes,
      mediumResImage: (map['image'] as String).mediumRes,
      highResImage: (map['image'] as String).highRes,
      imageURI: Uri.parse((map['image'] as String).mediumRes),
      playCount: int.parse(map['play_count']),
      year: int.parse(map['year']),
      permaURL: map['perma_url'],
      hasLyrics: hasLyrics,
      copyRightText: (moreInfo['copyright_text'] as String).sanitize,
      mediaURL: (moreInfo['encrypted_media_url'] as String).decryptUrl,
      duration: Duration(seconds: int.parse(moreInfo['duration'])),
      releaseDate: DateFormat('yy-mm-dd').parse(moreInfo['release_date']),
      previewMediaURL: moreInfo['vlink'],
      allArtists: allArtists,
      lyrics: hasLyrics ? await PluginFactory.getLyrics(map['id']) : null,
    );
  }

  static Artist createArtist(Map<String, dynamic> map) {
    return Artist(
      id: (map['id'] as String).sanitize,
      name: ((map['name'] ?? map['title']) as String).sanitize,
      role: PluginFactory.getArtistRole(map['role'] ?? map['type']),
      image: map['image'],
      permaURL: map['perma_url'],
      priority: map['position'],
    );
  }

  static ArtistRole getArtistRole(String role) {
    switch (role) {
      case 'primary_artists':
        return ArtistRole.primary;
      case 'featured_artists':
        return ArtistRole.fetaured;
      default:
        return ArtistRole.artist;
    }
  }

  static Future<String> getLyrics(String songId) async {
    return await cache.remember(
      'lyrics$songId',
      () async => jsonDecode(await http.get(
        Uri.parse(lyriclURL(songId)),
        headers: {"Accept": "application/json"},
      ).then((value) => value.body.split('-->')[1]))['lyrics']
          .toString()
          .replaceAll('<br>', '\n')
          .sanitize,
    );
  }
}
